package com.consultingwerk.plugins.smartunitlaunchconfig.preferences;

import java.net.MalformedURLException;
import java.net.URL;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Link;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.consultingwerk.plugins.common.ui.components.OpenFileFieldEditor;
import com.consultingwerk.plugins.common.ui.components.ProjectResourceFieldEditor;
import com.consultingwerk.plugins.common.ui.components.StringFieldEditor;
import com.consultingwerk.plugins.common.utilities.ConsoleHelper;
import com.consultingwerk.plugins.common.utilities.PreferencesHelper;
import com.consultingwerk.plugins.smartunitlaunchconfig.SmartUnitAttributes;

import org.eclipse.ui.IWorkbench;

/**
 * Preference Page to set default values for SmartUnit launch configurations and shortcuts
 */
public class SmartUnitPage
	extends FieldEditorPreferencePage
	implements IWorkbenchPreferencePage 
{
    /**
     * Constructor for the SmartUnitPage class
     */
	public SmartUnitPage() 
	{
		super(GRID);
		setPreferenceStore(PreferencesHelper.getStore(SmartUnitAttributes.Store));
		setDescription("Settings for SmartUnit tests");
	}
	
	/**
	 * Creates the field editors. 
	 */
	public void createFieldEditors() 
	{
	    addField (new OpenFileFieldEditor (getFieldEditorParent(), 
                  "ANT Executable:", 
                  SmartUnitAttributes.Store, 
                  SmartUnitAttributes.ANT, 
                  "Choose ANT Executable:", 
                  new String[] {"*.bat", "*.sh", "*.*"}, 
                  new String[] {"Batch Files", "Shell script Files", "All Files"}, 
                  OpenFileFieldEditor.VALIDATE_ON_CHANGED, 
                  ""));
	    
	    addField (new ProjectResourceFieldEditor (getFieldEditorParent(), 
	                                              "Build File:", 
	                                              SmartUnitAttributes.Store, 
	                                              SmartUnitAttributes.BuildFile, 
	                                              "Choose BuildFile:", 
	                                              ResourcesPlugin.getWorkspace().getRoot(), 
	                                              OpenFileFieldEditor.VALIDATE_ON_CHANGED, 
	                                              "unittest-runner.xml"));
	    
	    addField (new ProjectResourceFieldEditor (getFieldEditorParent(), 
                                                  "Output File:", 
                                                  SmartUnitAttributes.Store, 
                                                  SmartUnitAttributes.Output, 
                                                  "Choose Output:", 
                                                  ResourcesPlugin.getWorkspace().getRoot(), 
                                                  OpenFileFieldEditor.NO_VALIDATION, 
                                                  "results.xml"));
	    
	    addField (new StringFieldEditor (getFieldEditorParent(), 
	                                     "Test Class ANT Target:", 
	                                     SmartUnitAttributes.Store, 
	                                     SmartUnitAttributes.TestClassTarget, 
	                                     "")); 
	    
        addField (new StringFieldEditor (getFieldEditorParent(),
                                         "Scenario ANT Target:",
                                         SmartUnitAttributes.Store,
                                         SmartUnitAttributes.ScenarioTarget,
                                         ""));
        
	    addField (new StringFieldEditor (getFieldEditorParent(),
	                                     "Services:",
	                                     SmartUnitAttributes.Store,
	                                     SmartUnitAttributes.Services,
	                                     ""));
	    
	    addField(new StringFieldEditor (getFieldEditorParent(), 
	                                    "Additional ANT Arguments:", 
	                                    SmartUnitAttributes.Store, 
	                                    SmartUnitAttributes.AntArguments, 
	                                    ""));
	    
        addField(new BooleanFieldEditor (SmartUnitAttributes.UseConsole,
                                         "Output in console",
                                         getFieldEditorParent()));
        
        addField(new BooleanFieldEditor (SmartUnitAttributes.UseProfiler,
                                         "Enable Profiler",
                                         getFieldEditorParent()));

        String url = "https://consultingwerk.atlassian.net/wiki/spaces/SCL/pages";
        Link link = new Link (getFieldEditorParent(),
                              SWT.NONE);
        link.setText("<A href=\"" + url + "\">Confluence</A>");
        link.setLayoutData(this.getControl().getLayoutData());
        link.addSelectionListener(new SelectionAdapter () 
        {
            @Override
            public void widgetSelected (final SelectionEvent event)
            {               
                try 
                {
                    if(event.text.startsWith("https"))
                        PlatformUI.getWorkbench().getBrowserSupport().getExternalBrowser().openURL(new URL(url));
                } catch (PartInitException | MalformedURLException e) 
                {
                    ConsoleHelper.printException(e);
                }
            }
        });
	}

	/**
	 * Returns the preference store used
	 */
    @Override
    protected IPreferenceStore doGetPreferenceStore() 
    {
        return PreferencesHelper.getStore(SmartUnitAttributes.Store);
    }

    /**
     * Saves the preferences
     */
    @Override
    public boolean performOk() 
    {
        return super.performOk() && PreferencesHelper.save(SmartUnitAttributes.Store);
    }

    /**
     * Loads the preferences
     */
	public void init(IWorkbench workbench) 
	{
        PreferencesHelper.load(SmartUnitAttributes.Store);
	}
	
}