package com.consultingwerk.plugins.smartunitlaunchconfig.ui.tabs;

import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.consultingwerk.plugins.common.ui.components.BaseLaunchConfigurationTab;
import com.consultingwerk.plugins.common.ui.components.FileLookupListener;
import com.consultingwerk.plugins.common.ui.components.ProjectResourceLookupListener;
import com.consultingwerk.plugins.common.utilities.ConsoleHelper;
import com.consultingwerk.plugins.common.utilities.PreferencesHelper;
import com.consultingwerk.plugins.smartunitlaunchconfig.SmartUnitAttributes;

/**
 * Tab for defined SmartUnit launch configurations
 */
public class SmartUnitTab 
    extends BaseLaunchConfigurationTab 
{
    private final String name = "SmartUnit";
    
    private Text text_ant;
    private Text text_buildFile;
    private Text text_output;
    private Text text_testclasstarget;
    private Text text_services;
    private Text text_args;
    private Button btnUseConsole;
    private Button btnUseProfiler;
    private Text text_scenariotarget;
    private Text text_TestFile;
    private Text text_TestMethod;
    
    private int type;
    public static final int TestClass = 1;
    public static final int ScenarioTest = 2;
    
    public SmartUnitTab (int type)
    {
        if (   type == SmartUnitTab.TestClass
            || type == SmartUnitTab.ScenarioTest)
            this.type = type;
        else
            throw new IllegalArgumentException("Invalid type:_" + type);
    }
    
    /**
     * Creates the Tabs control
     * @param parent The parent Composite
     * @wbp.parser.entryPoint
     */
    @Override
    public void createControl (Composite parent) 
    {
        Composite comp = new Group (parent, 
                                    SWT.NONE);
        this.setControl(comp);
        comp.setLayout(new GridLayout(3, false));
        
        Label lblAnt = new Label(comp, SWT.RIGHT);
        lblAnt.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblAnt.setText("ANT Executable:");
        
        text_ant = new Text(comp, SWT.BORDER);
        text_ant.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        text_ant.addModifyListener(this);
        
        Button button_ant = new Button(comp, SWT.NONE);
        button_ant.setText("...");
        button_ant.addSelectionListener (new FileLookupListener (text_ant,
                                                                 "Select ANT Executable",
                                                                 new String[] {"*.bat", "*.cmd", "*.*"},
                                                                 new String[] {"Batch Files", "Command Files", "All Files"}));
        
        Label lblTestfile = new Label(comp, SWT.NONE);
        lblTestfile.setAlignment(SWT.RIGHT);
        lblTestfile.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblTestfile.setText("Test File:");
        
        text_TestFile = new Text(comp, SWT.BORDER);
        text_TestFile.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        text_TestFile.addModifyListener(this);
        
        Button button_TestFile = new Button(comp, SWT.NONE);
        button_TestFile.setText("...");
        if (this.type == SmartUnitTab.TestClass)
            button_TestFile.addSelectionListener (new ProjectResourceLookupListener (text_TestFile,
                                                                                     "Select test class",
                                                                                     ResourcesPlugin.getWorkspace().getRoot(),
                                                                                     false));
        else if (this.type == SmartUnitTab.ScenarioTest)
            button_TestFile.addSelectionListener (new ProjectResourceLookupListener (text_TestFile,
                                                                                     "Select test scenario",
                                                                                     ResourcesPlugin.getWorkspace().getRoot(),
                                                                                     false));
        
        if (this.type == SmartUnitTab.TestClass)
        {
            Label lblTestMethod = new Label(comp, SWT.NONE);
            lblTestMethod.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
            lblTestMethod.setText("Test Method:");
            
            text_TestMethod = new Text(comp, SWT.BORDER);
            text_TestMethod.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
            text_TestMethod.addModifyListener(this);
            new Label(comp, SWT.NONE);
        }
        
        Label lblBuildFile = new Label(comp, SWT.RIGHT);
        lblBuildFile.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblBuildFile.setText("Build File:");
        
        text_buildFile = new Text(comp, SWT.BORDER);
        text_buildFile.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        text_buildFile.addModifyListener(this);
        
        Button button_buildFile = new Button(comp, SWT.NONE);
        button_buildFile.setText("...");
        button_buildFile.addSelectionListener(new ProjectResourceLookupListener (text_buildFile,
                                                                                 "Select build-file",
                                                                                 ResourcesPlugin.getWorkspace().getRoot(),
                                                                                 false));    
        
        Label lblOutput = new Label(comp, SWT.NONE);
        lblOutput.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblOutput.setAlignment(SWT.RIGHT);
        lblOutput.setText("Output:");
        
        text_output = new Text(comp, SWT.BORDER);
        text_output.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        text_output.addModifyListener(this);
        
        Button button_output = new Button(comp, SWT.NONE);
        button_output.setText("...");
        button_output.addSelectionListener(new ProjectResourceLookupListener (text_output,
                                                                              "Select output file",
                                                                              ResourcesPlugin.getWorkspace().getRoot(),
                                                                              false));    
        
        if (this.type == SmartUnitTab.TestClass)
        {
            Label lblTarget = new Label(comp, SWT.NONE);
            lblTarget.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
            lblTarget.setAlignment(SWT.RIGHT);
            lblTarget.setText("ANT Target:");
            
            text_testclasstarget = new Text(comp, SWT.BORDER);
            text_testclasstarget.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
            text_testclasstarget.addModifyListener(this);
            new Label(comp, SWT.NONE);
        }
        else if (this.type == SmartUnitTab.ScenarioTest)
        {
            Label lblScenariotarget = new Label(comp, SWT.NONE);
            lblScenariotarget.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
            lblScenariotarget.setAlignment(SWT.RIGHT);
            lblScenariotarget.setText("ANT Target:");
            
            text_scenariotarget = new Text(comp, SWT.BORDER);
            text_scenariotarget.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
            text_scenariotarget.addModifyListener(this);
            new Label(comp, SWT.NONE);
        }
        
        Label lblServices = new Label(comp, SWT.NONE);
        lblServices.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblServices.setAlignment(SWT.RIGHT);
        lblServices.setText("Services:");
        
        text_services = new Text(comp, SWT.BORDER);
        text_services.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        text_services.addModifyListener(this);
        new Label(comp, SWT.NONE);
        
        Label lblArguments = new Label(comp, SWT.NONE);
        lblArguments.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        lblArguments.setAlignment(SWT.RIGHT);
        lblArguments.setText("Additional ANT Arguments:");
        
        text_args = new Text(comp, SWT.BORDER);
        text_args.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
        text_args.addModifyListener(this);
        new Label(comp, SWT.NONE);
        new Label(comp, SWT.NONE);
        
        btnUseConsole = new Button(comp, SWT.CHECK);
        btnUseConsole.setText("Use Console");
        btnUseConsole.addSelectionListener(this);
        new Label(comp, SWT.NONE);
        new Label(comp, SWT.NONE);
        
        btnUseProfiler = new Button(comp, SWT.CHECK);
        btnUseProfiler.setText("Use Profiler");
        btnUseProfiler.addSelectionListener(this);
        new Label(comp, SWT.NONE);
    }

    /**
     * The tabs name
     */
    @Override
    public String getName() 
    {
        return this.name;
    }

    /**
     * Initializes the tab from an existing launch configuration
     */
    @Override
    public void initializeFrom(ILaunchConfiguration config) 
    {
        try
        {
            this.initText  (text_ant,       config, SmartUnitAttributes.ANT,          "");
            this.initText  (text_buildFile, config, SmartUnitAttributes.BuildFile,    "");
            this.initText  (text_output,    config, SmartUnitAttributes.Output,       "");
            this.initText  (text_args,      config, SmartUnitAttributes.AntArguments, "");
            this.initText  (text_services,  config, SmartUnitAttributes.Services,     "");
            this.initButton(btnUseConsole,  config, SmartUnitAttributes.UseConsole,   true);
            this.initButton(btnUseProfiler, config, SmartUnitAttributes.UseProfiler,  false);
            this.initText  (text_TestFile,  config, SmartUnitAttributes.TestFile,     "");
            if (this.type == SmartUnitTab.TestClass)
            {
                this.initText (text_TestMethod,      config, SmartUnitAttributes.TestMethod,      "");
                this.initText (text_testclasstarget, config, SmartUnitAttributes.TestClassTarget, "");
            }
            else if (this.type == SmartUnitTab.ScenarioTest)
                this.initText (text_scenariotarget,  config, SmartUnitAttributes.ScenarioTarget,  "");
        }
        catch (CoreException e)
        {
            ConsoleHelper.printException(e);
        }
    }
    
    /**
     * Initializes a text
     * @param text The text to set
     * @param config The config to set from
     * @param attr The name of the attribute
     * @param def The default value
     * @throws CoreException
     */
    protected void initText (Text text, 
                             ILaunchConfiguration config, 
                             String attr, 
                             String def) 
                             throws CoreException
    {
        text.setText (config.getAttribute (attr, def));
    }
    
    /**
     * Initializes a button checked state
     * @param btn The button to set
     * @param config The config to set from
     * @param attr The name of the attribute
     * @param def The default value
     * @throws CoreException
     */
    protected void initButton (Button btn,
                               ILaunchConfiguration config,
                               String attr,
                               boolean def)
                               throws CoreException
    {
        btn.setSelection(config.getAttribute(attr, def));
    }

    /**
     * Applies the current values to a given configuration
     * @param config The configuration
     */
    @Override
    public void performApply (ILaunchConfigurationWorkingCopy config) 
    {
        this.applyText  (text_ant,       config, SmartUnitAttributes.ANT);
        this.applyText  (text_buildFile, config, SmartUnitAttributes.BuildFile);
        this.applyText  (text_output,    config, SmartUnitAttributes.Output);
        this.applyText  (text_args,      config, SmartUnitAttributes.AntArguments);
        this.applyText  (text_services,  config, SmartUnitAttributes.Services);
        this.applyButton(btnUseConsole,  config, SmartUnitAttributes.UseConsole);
        this.applyButton(btnUseProfiler, config, SmartUnitAttributes.UseProfiler);
        this.applyText  (text_TestFile,  config, SmartUnitAttributes.TestFile);
        if (this.type == SmartUnitTab.TestClass)
        {
            this.applyText (text_TestMethod,      config, SmartUnitAttributes.TestMethod);
            this.applyText (text_testclasstarget, config, SmartUnitAttributes.TestClassTarget);
        }
        else if (this.type == SmartUnitTab.ScenarioTest)
            this.applyText (text_scenariotarget,  config, SmartUnitAttributes.ScenarioTarget);
    }
    
    /**
     * Applies a given texts value to a given config
     * @param text The text to set from
     * @param config The config to set it
     * @param attr The name of the attribute to set
     */
    protected void applyText (Text text, 
                              ILaunchConfigurationWorkingCopy config, 
                              String attr)
    {
        config.setAttribute(attr, text.getText());
    }
    
    /**
     * Applies a given buttons value to a given config
     * @param btn The button to set from
     * @param config The config to set in
     * @param attr The name of the attribute
     */
    protected void applyButton (Button btn,
                                ILaunchConfigurationWorkingCopy config, 
                                String attr)
    {
    config.setAttribute(attr, btn.getSelection());
    }

    /**
     * Fills a config with default values
     * @param The config to fill
     */
    @Override
    public void setDefaults(ILaunchConfigurationWorkingCopy config) 
    {
        this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.ANT);
        this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.BuildFile);
        this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.Output);
        this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.AntArguments);
        this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.Services);
        this.defaultBool   (config, SmartUnitAttributes.Store, SmartUnitAttributes.UseConsole);
        this.defaultBool   (config, SmartUnitAttributes.Store, SmartUnitAttributes.UseProfiler);
        this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.TestFile);
        if (this.type == SmartUnitTab.TestClass)
        {
            this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.TestMethod);
            this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.TestClassTarget);
        }
        else if (this.type == SmartUnitTab.ScenarioTest)
            this.defaultString (config, SmartUnitAttributes.Store, SmartUnitAttributes.ScenarioTarget);
    }
    
    /**
     * Fills a string default
     * @param config The config to fill
     * @param store The store to get the default preference from
     * @param attr The name of the attribute
     */
    protected void defaultString (ILaunchConfigurationWorkingCopy config, 
                                  String store, 
                                  String attr)
    {
        config.setAttribute(attr, PreferencesHelper.getStringPreference(store, attr));
    }
    
    /**
     * Fills a boolean default
     * @param config The config to fill
     * @param store The store to get the default preference from
     * @param attr The name of the attribute
     */
    protected void defaultBool (ILaunchConfigurationWorkingCopy config, 
                                String store, 
                                String attr)
    {
        config.setAttribute(attr, PreferencesHelper.getBoolPreference(store, attr));
    }

}
