package com.consultingwerk.plugins.smartunitlaunchconfig.ui.views;

import java.awt.Toolkit;
import java.awt.datatransfer.StringSelection;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.EditorPart;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.consultingwerk.plugins.common.assertions.FileAssert;
import com.consultingwerk.plugins.common.assertions.ObjectAssert;
import com.consultingwerk.plugins.common.ui.components.ValueDisplayComposite;
import com.consultingwerk.plugins.common.utilities.ConsoleHelper;
import com.consultingwerk.plugins.common.utilities.FileHelper;
import com.consultingwerk.plugins.smartunitlaunchconfig.SmartUnitAttributes;

import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.swt.widgets.Widget;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.layout.FillLayout;
import swing2swt.layout.BorderLayout;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;

/**
 * Shows the results of a SmartUnit test
 */
public class SmartUnitView 
    extends EditorPart
    implements SelectionListener
{
    private String filename;
    
    private boolean initialized = false;
    
    private ValueDisplayComposite compRuns;
    private ValueDisplayComposite compErrors;
    private ValueDisplayComposite compFailures;
    private Tree tree;
    private Text text;
    private Composite composite;
    
    private boolean isDisposed = false;
    
    private HashMap<String, HashMap<String, String>> map;
    private SashForm sashForm;
    private Button btnCopyToClipboard;
    private Label label;
    private Composite composite_3;
    private Label label_1;

    /**
     * Constructor for the SmartUnitView class
     */
    public SmartUnitView ()
    {
        map = new HashMap<String, HashMap<String, String>>();
    }
    
    /**
     * This view does not save
     */
    @Override
    public void doSave (IProgressMonitor monitor) 
    { /* NOOP */}

    /**
     * This view does not save
     */
    @Override
    public void doSaveAs () 
    { /* NOOP */ }

    /**
     * Initializes the view
     * @param The editor site to init with
     * @param The editor input to init with
     * @throws PartInitException
     */
    @Override
    public void init (IEditorSite site, 
                      IEditorInput input) 
                throws PartInitException 
    {
        setSite(site);
        setInput(input);

        this.filename = FileHelper.getFilename(this);
        if (this.initialized)
            this.parseResultsFile ();
    }
    
    /**
     * Opens the view for the given file
     * @param filename The file to open the view for
     */
    public void openFile (String filename)
    {
        this.filename = filename;

        this.parseResultsFile();
    }
    
    /**
     * Disposes the trees items
     */
    private void disposeTreeItems ()
    {
        for (TreeItem item: this.tree.getItems())
        {
            this.disposeTreeItems(item);
            item.dispose();
        }
    }
    
    /**
     * Disposes the given items items recursively
     * @param item The item whose items should be disposed
     */
    private void disposeTreeItems (TreeItem item)
    {
        for (TreeItem itemsItem: item.getItems())
            this.disposeTreeItems(itemsItem);
    }

    /**
     * The view doesn't change the file, therefore can't be dirty
     */
    @Override
    public boolean isDirty() 
    {
        return false;
    }

    /**
     * No reason to save
     */
    @Override
    public boolean isSaveAsAllowed() 
    {
        return false;
    }

    /**
     * Creates the control
     * @param parent The parent Composite
     */
    @Override
    public void createPartControl(Composite parent) 
    {
        parent.setLayout(new FillLayout(SWT.HORIZONTAL));
        
        sashForm = new SashForm(parent, SWT.NONE);
        sashForm.setTouchEnabled(true);
        
        composite = new Composite(sashForm, SWT.NONE);
        composite.setLayout(new BorderLayout(0, 0));
        
        Composite composite_2 = new Composite(composite, SWT.NONE);
        composite_2.setLayoutData(BorderLayout.NORTH);
        
        this.compRuns = new ValueDisplayComposite(composite_2, SWT.NONE);
        this.compRuns.setImage(FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/bullet_triangle_green.png"));
        this.compRuns.setText("Runs:");
        this.compRuns.setValue("0");
        
        this.compErrors = new ValueDisplayComposite(composite_2, SWT.NONE);
        this.compErrors.setImage(FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/bullet_square_red.png"));
        this.compErrors.setText("Errors:");
        this.compErrors.setValue("0");
        
        this.compFailures = new ValueDisplayComposite(composite_2, SWT.NONE);
        this.compFailures.setImage(FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/bullet_square_blue.png"));
        this.compFailures.setText("Failures:");
        this.compFailures.setValue("0");
        composite_2.setLayout(new GridLayout(4, false));
        
        label = new Label(composite_2, SWT.NONE);
        GridData gd_label = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
        gd_label.widthHint = 0;
        gd_label.heightHint = 25;
        label.setLayoutData(gd_label);
        
        this.tree = new Tree(composite, SWT.BORDER);
        tree.setLayoutData(BorderLayout.CENTER);
        this.tree.addSelectionListener((SelectionListener) this);
        
        Composite composite_1 = new Composite(sashForm, SWT.NONE);
        composite_1.setLayout(new BorderLayout(0, 0));
        
        this.text = new Text(composite_1, SWT.BORDER | SWT.READ_ONLY | SWT.H_SCROLL | SWT.V_SCROLL | SWT.CANCEL | SWT.MULTI);
        text.setLayoutData(BorderLayout.CENTER);
        text.setEditable(false);
        
        composite_3 = new Composite(composite_1, SWT.NONE);
        composite_3.setLayoutData(BorderLayout.NORTH);
        composite_3.setLayout(new GridLayout(3, false));
        
        Label lblNewLabel = new Label(composite_3, SWT.NONE);
        lblNewLabel.setText("Failure Trace:");
        
        label_1 = new Label(composite_3, SWT.NONE);
        GridData gd_label_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
        gd_label_1.heightHint = 25;
        label_1.setLayoutData(gd_label_1);
        
        btnCopyToClipboard = new Button(composite_3, SWT.RIGHT);
        btnCopyToClipboard.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
        btnCopyToClipboard.setText("Copy to Clipboard  ");
        sashForm.setWeights(new int[] {454, 453});
        btnCopyToClipboard.addSelectionListener(this);
     
        this.parseResultsFile ();
        this.initialized = true;
    }

    /**
     * Sets the focus on this control
     */
    @Override
    public void setFocus() 
    {
        PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().activate(this);
    }
    
    /**
     * Disposes the control
     */
    @Override
    public void dispose() 
    {
        super.dispose();
        this.isDisposed = true;
    }
    
    /**
     * Returns whether this control is disposed
     * @return Whether this control is disposed
     */
    public boolean isDisposed ()
    {
        return this.isDisposed;
    }

    /**
     * Empties the map of error-strings
     */
    private void emptyMap()
    {
        for(String key: this.map.keySet())
            map.remove(key);
        map = new HashMap<String, HashMap<String, String>>();
    }
    
    /**
     * Resets the components
     */
    protected void reset ()
    {
        this.emptyMap();
        this.text.setText("");
        this.disposeTreeItems();
        this.compRuns.setValue("0");
        this.compErrors.setValue("0");
        this.compFailures.setValue("0");
    }
    
    /**
     * Parses the current file and fills the tree and labels
     */
    private void parseResultsFile () 
    {
        try
        {
            ObjectAssert.isValid(this.filename, "filename");
            FileAssert.Exists(this.filename);
            FileAssert.CanRead(this.filename);
            ObjectAssert.isValid(this.compRuns, "compRuns");
            ObjectAssert.isValid(this.compErrors, "compErrors");
            ObjectAssert.isValid(this.compFailures, "compFailures");
            ObjectAssert.isValid(this.text, "text");
            ObjectAssert.isValid(this.tree, "tree");
            
            File f = new File(this.filename);
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document doc = db.parse(f);
            
            Element testSuiteElement;
            Element testCaseElement;
            Element errorElement;
            
            TreeItem testSuiteTreeItem;
            TreeItem testCaseTreeItem;
            
            double testSuiteTime;
            Integer tests = 0;
            Integer errors = 0;
            Integer failures = 0;
            boolean failed;
            String errorMessage;
            
            HashMap<String, String> suiteMap;
            
            doc.getDocumentElement().normalize();
            
            this.reset();
            
            for (Node testSuiteNode  = doc.getDocumentElement().getFirstChild(); 
                      testSuiteNode != null; 
                      testSuiteNode  = testSuiteNode.getNextSibling())
            {
                if(testSuiteNode instanceof Element)
                {
                    testSuiteElement = (Element) testSuiteNode;
                    
                    testSuiteTreeItem = new TreeItem (this.tree, SWT.NONE);
                    testSuiteTreeItem.setImage(new Image(this.tree.getDisplay(), FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/crash_test_dummy.png")));
                    
                    testSuiteTime = 0.0;
                    failed = false;
                    
                    suiteMap = new HashMap<String, String>();
                    
                    for (Node testCaseNode  = testSuiteNode.getFirstChild(); 
                              testCaseNode != null; 
                              testCaseNode  = testCaseNode.getNextSibling())
                    {
                        if(testCaseNode instanceof Element)
                        {
                            testCaseElement = (Element) testCaseNode;
                            
                            testCaseTreeItem = new TreeItem (testSuiteTreeItem, SWT.NONE);
                            testCaseTreeItem.setText(testCaseElement.getAttribute("name") + " (" + testCaseElement.getAttribute("time") + ")");
                            
                            if(testCaseElement.getAttribute("time").length() > 0)
                                testSuiteTime += Double.parseDouble(testCaseElement.getAttribute("time").replace(',', '.'));
                            
                            tests++;
                            if(testCaseNode.getFirstChild() != null)
                            {
                                errors++;
                                failed = true;
                                testCaseTreeItem.setImage(new Image(this.tree.getDisplay(), FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/error.png")));
                                for (Node errorNode  = testCaseNode.getFirstChild();
                                          errorNode != null;
                                          errorNode  = errorNode.getNextSibling())
                                {
                                        if(errorNode instanceof Element)
                                        {
                                            errorElement = (Element) errorNode;
                                        if (!suiteMap.containsKey(testCaseTreeItem.getText()))
                                            suiteMap.putIfAbsent(testCaseTreeItem.getText(), errorElement.getTextContent());
                                        else
                                        {
                                            errorMessage = suiteMap.get(testCaseTreeItem.getText());
                                            errorMessage += errorElement.getTextContent();
                                            suiteMap.put(testCaseTreeItem.getText(), errorMessage);
                                        }   
                                    }
                                }
                            }
                            else
                                testCaseTreeItem.setImage(new Image(this.tree.getDisplay(), FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/ok.png")));
                        }
                        
                    }
                    testSuiteTreeItem.setText(testSuiteElement.getAttribute("name") + " (" + String.format("%.3f", testSuiteTime).replace(',', '.') + ")");
                    
                    this.map.put(testSuiteTreeItem.getText(), suiteMap);
                    
                    if(failed)
                        testSuiteTreeItem.setImage(new Image(this.tree.getDisplay(), FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/error.png")));
                    else
                        testSuiteTreeItem.setImage(new Image(this.tree.getDisplay(), FileHelper.findPluginFile(SmartUnitAttributes.PluginID, "icons/ok.png")));
                    testSuiteTreeItem.setExpanded(true);
                }
            }
            this.compRuns.setValue(tests.toString());
            this.compErrors.setValue(errors.toString());
            this.compFailures.setValue(failures.toString());
            
            tree.update();
        }
        catch (SAXException | IOException | ParserConfigurationException e)
        {
            ConsoleHelper.printException(e);
        }
    }

    /**
     * Handler for the SelectionEvent
     * @param arg The event
     */
    @Override
    public void widgetDefaultSelected(SelectionEvent arg) 
    {
        this.SelectionEventHandler(arg);
    }

    /**
     * Handler for the SelectionEvent
     * @param arg The event
     */
    @Override
    public void widgetSelected(SelectionEvent arg)
    {
        this.SelectionEventHandler(arg);
    }
    
    /**
     * Handler for the SelectionEvent event of buttons and tree items
     * @param arg
     */
    private void SelectionEventHandler (SelectionEvent arg)
    {
        switch(arg.widget.getClass().getName())
        {
            case "org.eclipse.swt.widgets.Tree":
                this.putErrorText(arg.item);
                break;
            case "org.eclipse.swt.widgets.Button":
                this.copyToClipboard();
                break;
        }
    }
    
    /**
     * Puts the error-text into the text widget if the selected TreeItem is a failed test
     * @param arg The selected widget
     */
    private void putErrorText (Widget arg)
    {
        TreeItem ti;
        
        this.text.setText ("");
        
        if(arg instanceof TreeItem)
        {
            ti = (TreeItem) arg;
            
            if (   ti.getParentItem () != null
                && ti.getParentItem () instanceof TreeItem)
            {
                if (this.map.containsKey (ti.getParentItem ().getText ()))
                    if (this.map.get (ti.getParentItem ().getText ()).containsKey (ti.getText ()))
                        this.text.setText (this.map.get (ti.getParentItem ().getText ()).get (ti.getText ()));
            }
        }
    }
    
    private void copyToClipboard ()
    {
        Toolkit.getDefaultToolkit ()
               .getSystemClipboard ()
               .setContents (new StringSelection (this.text.getText ()), 
                             null);
    }
}
