package com.consultingwerk.plugins.smartunitlaunchconfig.utilities;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.swt.widgets.Display;

import com.consultingwerk.plugins.common.assertions.FileAssert;
import com.consultingwerk.plugins.common.utilities.AntHelper;
import com.consultingwerk.plugins.common.utilities.ConsoleHelper;
import com.consultingwerk.plugins.common.utilities.FileHelper;
import com.consultingwerk.plugins.smartunitlaunchconfig.SmartUnitAttributes;
import com.consultingwerk.plugins.smartunitlaunchconfig.ui.views.SmartUnitViewLauncher;

/**
 * Provides a method to run SmartUnit tests
 */
public class SmartUnitHelper 
{
    /**
     * Runs a SmartUnit build.xml
     * @param launch The Launch object
     * @param mode The mode to run the config in
     * @throws CoreException
     * @throws IOException
     * @throws InterruptedException 
     */
    public static void run (ILaunch launch,
                            String mode) 
                            throws CoreException, IOException, InterruptedException
    {
        ILaunchConfiguration config = launch.getLaunchConfiguration();
        
        String  ant;
        String  buildFile;
        String  target;
        String  testFile;
        String  testMethod;
        String  outputFile;
        boolean console;
        String  services;
        boolean profiler;
        String  jre;
        String  arguments;
        
        AntHelper antHelper;
        Process proc;
        
        ant        = config.getAttribute (SmartUnitAttributes.ANT,          "");
        buildFile  = config.getAttribute (SmartUnitAttributes.BuildFile,    "");
        testFile   = config.getAttribute (SmartUnitAttributes.TestFile,     "");
        testMethod = config.getAttribute (SmartUnitAttributes.TestMethod,   "");
        outputFile = config.getAttribute (SmartUnitAttributes.Output,       "");
        console    = config.getAttribute (SmartUnitAttributes.UseConsole,   true);
        services   = config.getAttribute (SmartUnitAttributes.Services,     "");
        profiler   = config.getAttribute (SmartUnitAttributes.UseProfiler,  false);
        jre        = config.getAttribute (SmartUnitAttributes.JRE,          "");
        arguments  = config.getAttribute (SmartUnitAttributes.AntArguments, "");
        if(testFile.endsWith(".cls"))
            target = config.getAttribute (SmartUnitAttributes.TestClassTarget, "");
        else if(testFile.endsWith(".scenario"))
            target = config.getAttribute (SmartUnitAttributes.ScenarioTarget,  "");
        else
            target = "";
        
        FileAssert.Exists (ant);
        FileAssert.CanExecute (ant);
        
        FileAssert.Exists (buildFile);
        FileAssert.CanRead (buildFile);
        
        antHelper = new AntHelper (ant, 
                                   buildFile, 
                                   target);
        
        FileAssert.Exists (testFile);
        FileAssert.CanRead (testFile);
        FileAssert.CanExecute (testFile);
        
        if(testFile.endsWith(".cls"))
            antHelper.addArg ("smartunit.testclass", FileHelper.FileNameToClassName(testFile));
        else if(testFile.endsWith(".scenario"))
            antHelper.addArg ("smartunit.testscenario", testFile);
        if (testMethod != null && testMethod.length() > 0)
            antHelper.addArg ("smartunit.testmethod", testMethod);
        else
            antHelper.addArg ("smartunit.testmethod", "");
        
        antHelper.addArg ("smartunit.output", outputFile);
        
        if (services != null && services.length() > 0)
            antHelper.addArg ("smartunit.services", services);
        else
            antHelper.addArg ("smartunit.services", "");
        
        if(profiler)
            antHelper.addArg ("smartunit.profiler", "true");
        else
            antHelper.addArg ("smartunit.profiler", "false");
        
        if (jre != null && jre.length() > 0)
            antHelper.addArg ("smartunit.jre", jre);
        else
            antHelper.addArg ("smartunit.jre", "");

        if (arguments != null && arguments.length() > 0)
            antHelper.addArg (arguments);
   
        proc = antHelper.run ();
        
        Display display = Display.getCurrent();
        if(console)
        {
            ConsoleHelper.findConsole("SmartUnit").activate();
            Thread consoleThread = new Thread(() -> 
            {
                try 
                {
                    BufferedReader out = new BufferedReader(new InputStreamReader(proc.getInputStream()));
                    
                    String line;
                    
                    while((line = out.readLine()) != null)
                        ConsoleHelper.println("SmartUnit", line);
                    display.asyncExec(new Thread(() ->
                    {
                        SmartUnitViewLauncher.openFile(outputFile);
                    }));
                }
                catch (IOException e) 
                {
                    ConsoleHelper.printException(e);
                }
            });
            consoleThread.setDaemon(true);
            consoleThread.start();
        } 
        else
        {
            try 
            {
                proc.waitFor();
                SmartUnitViewLauncher.openFile(outputFile);
            } 
            catch (InterruptedException e) 
            {
                ConsoleHelper.printException(e);
            }
        }
    }
}
